/*
        PPPCOSTS, version 0.03b2. Calculates PPP connection phone costs.
        
        Copyright (c) 1996 Tillmann Steinbrecher.
        May be distributed according to the terms of the GNU
        General Public License version 2. No warranty.

	Ver 0.01 initial version
	Ver 0.02 bugfixes. revamped calculation routine. public release.
	Ver 0.03 Added "next in xx:xx" display. Idea & Code by
		 Didier.Chalm@infini.fr. Added optional argument
		 "-f filename" to specify a different pppcosts-file. 
		 Idea & code by tandlerj@cips01.physik.uni-bonn.de
		 Added a few #defines for easier localization.
		 	
	Ver x.xx Whatever YOU suggest (or implement :) )

	Please send your comments to tst@bigfoot.com
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <time.h>

#include "costs.c"

#define PPP_PID_FILE "/var/run/ppp0.pid"

#define TOTAL_FILE ".pppcosts"

#define verbose 0
#define delay_secs 1

void main(int argc, char *argv[]){

  FILE *pidfile, *statfile, *uptimefile, *totfile;
  char pid_chr[12], uptime_chr[64],tottime_chr[32], totunits_chr[32];
  char *stat_chr, totfilename[64];
  char statfile_name[64];
  char count;
  char rem;
  long starttime, uptime, online_secs,lasttime,units,next_unit;
  long tottime, totunits, bdtime;
  float costs;
  struct tm* loctim;
     
 printf("\nPPPCOSTS 0.03 (c) T. Steinbrecher\n");
 
 sprintf(totfilename,"%s/" TOTAL_FILE, getenv("HOME"));
  
 if(argc>1){
  if (!strcmp(argv[1],"-f"))
	    {
	     if(argc<3)
	     	fprintf(stderr, "Warning: no filename specified with '-f' option, using default.\n");
	     else{	
	      sprintf(totfilename,"%s", argv[2]);
	      fprintf(stderr,"Using cost/time file: %s\n",totfilename);
	     }	
            }
    else fprintf(stderr,"Warning: ignoring invalid command line option '%s'\n",argv[1]); 
  }
 
 
  if(!(totfile=fopen(totfilename, "rt"))) 
  	fprintf(stderr, "Warning: cost/time file %s not present\n", totfilename);

  fgets(tottime_chr,32,totfile);
  fgets(totunits_chr,32,totfile);
  fclose(totfile);
  tottime=strtol(tottime_chr,NULL);
  totunits=strtol(totunits_chr,NULL);

  /* Notice:                             */					  bdtime=time(0);              
  /* These lines here are very important */					  loctim=localtime(&bdtime);
										  if(loctim->tm_mon==3 && loctim->tm_mday==18)
  /* So please don't remove them ;-) ... */					   	fprintf(stderr, "April 18: Send 'Happy Birthday' to tst@bigfoot.com !\a\n");

for(;;){ 

#ifdef CURRENCY_AFTER_COST
  printf("Total online: %.2ld:%.2ld:%.2ld (%ld units, %." DECIMALS "f" CURRENCY ")\n", 	
   tottime / 3600 , (tottime % 3600) / 60, tottime % 60, totunits, totunits * COSTS_PER_UNIT);
#else
  printf("Total online: %.2ld:%.2ld:%.2ld (%ld units, " CURRENCY "%." DECIMALS "f)\n", 	
   tottime / 3600 , (tottime % 3600) / 60, tottime % 60, totunits, totunits * COSTS_PER_UNIT);
#endif          

  while( ! (pidfile = fopen(PPP_PID_FILE,"rt")) ){
       fprintf(stderr, "Offline.                           \r");
       sleep(delay_secs);
       }

  fgets(pid_chr,12,pidfile);
  fclose(pidfile);
  pid_chr[strlen(pid_chr)-1]='\0';
  strcpy(statfile_name,"/proc/");
  strcat(statfile_name,pid_chr);
  strcat(statfile_name,"/stat");
  if(verbose) printf("Now online - PPP daemon PID: %s\n",pid_chr);
  if(verbose) printf("Opening stat file: %s\n", statfile_name);
  statfile=fopen(statfile_name,"rt");
  
  stat_chr=malloc(512);
  
  fgets(stat_chr,512,statfile);
  fclose(statfile);
  if(verbose)printf(stat_chr);
  for(count=0;count<21;count++){
    stat_chr=index(stat_chr,32)+1;   /* Skip first 21 entries */ 
   }
  stat_chr[index(stat_chr,32)-stat_chr]=0; 
  if(verbose) printf("After truncating: ");
  starttime=strtol(stat_chr,NULL);
  starttime=starttime / 100;
  if(verbose) printf("ppp daemon running since uptime=%ld secs\n\n",starttime);

  lasttime=0; 	
  units=0;
    
	while( (statfile=fopen(statfile_name, "rt"))){
	   fclose(statfile);
	   uptimefile = fopen("/proc/uptime","rt");
	   fgets(uptime_chr,12,uptimefile);
	   fclose(uptimefile);
	   uptime_chr[index(uptime_chr,'.')-uptime_chr]=0;
	   uptime=strtol(uptime_chr,NULL);
	   if(verbose) printf("System running for %ld secs, ",uptime);
	   online_secs=uptime-starttime;

/* Be careful when trying to modify this calculation routine, as it is
   is hard to understand and easy to misunderstand */

	   	for(	lasttime;
	   		lasttime<=online_secs;
	   		lasttime+=getunitlength(time(0)-online_secs+lasttime ))
	   		units++;

next_unit=lasttime-online_secs;

#ifdef CURRENCY_AFTER_COST
  fprintf(stderr, "ONLINE: %.2ld:%.2ld:%.2ld (%u units, %." DECIMALS "f" CURRENCY ", next in %.2ld:%.2ld) \r", 
  	online_secs / 3600 , (online_secs % 3600) / 60, online_secs % 60, 
  	units, units * COSTS_PER_UNIT, next_unit / 60, next_unit % 60);
#else
  fprintf(stderr, "ONLINE: %.2ld:%.2ld:%.2ld (%u units, " CURRENCY "%." DECIMALS "f, next in %.2ld:%.2ld) \r", 
  	online_secs / 3600 , (online_secs % 3600) / 60, online_secs % 60, 
  	units, units * COSTS_PER_UNIT, next_unit / 60, next_unit % 60);
#endif

  sleep(delay_secs);
  }
  fprintf(stderr,"\n");
  
  tottime+=online_secs;
  totunits+=units;

  if((totfile=fopen(totfilename,"wt"))==0)
  	fprintf(stderr, "Warning: can't save costs/time to %s\n", totfilename );
  else {
	  fprintf(totfile,"%ld\n", tottime);
	  fprintf(totfile,"%ld\n", totunits);
	  fclose(totfile);
      }	
  }
  printf("\n\n");
  }
